import os
import imp
from importlib import import_module

from flask.ext.testing import TestCase

from alembic import command
from alembic.config import Config as AlembicConfig

class Config(AlembicConfig):
  def get_template_directory(self):
    package_dir = os.path.abspath(os.path.dirname(__file__))
    return os.path.join(package_dir, 'templates')

class ResourceTest(TestCase):

  finny_app = None

  def setUp(self):
    command.upgrade(self.config, "head")

  def tearDown(self):
    self.db.session.remove()
    self.db.drop_all()

    self.db.engine.execute("drop schema public cascade; create schema public;")

  def _get_config(self, current_app):
    directory = current_app.extensions['migrate'].directory

    config = Config(os.path.join(directory, 'alembic.ini'))
    config.set_main_option('script_location', directory)
    return config

  def create_app(self):
    if ResourceTest.finny_app:
      return ResourceTest.finny_app

    current_path = os.getcwd()

    config = imp.new_module('config')
    config.__file__ = "config"
    # loads the configuration file
    with open(current_path + "/__init__.py") as config_file:
      exec(compile(config_file.read(), "config", 'exec'), config.__dict__)

    finny_app = config.__APP__

    boot_module = import_module("%s.boot" % finny_app)

    create_app = boot_module.create_app
    ResourceTest.db = boot_module.db

    current_app = create_app("andromeda_api", "test", "default")

    ResourceTest.config = self._get_config(current_app)
    ResourceTest.finny_app = current_app

    return current_app



import os

from jinja2 import Environment, PackageLoader
from finny.command import Command

BASE_FOLDER_TEMPLATES = [
  ".gitignore",
  "requirements.txt",
  "README.md",
  "manage.py",
  "__init__.py"
]

CONFIG_INITIALIZERS_TEMPLATES = [ "app.py" ]
CONFIG_RUNNERS_TEMPLATES = [ "default.py" ]
CONFIG_TEMPLATES = [
  "boot.py",
  "development.py.sample",
  "test.py.sample",
  "production.py.sample"
]

class GenerateStructure(Command):

  def __init__(self):
    pass

  def _touch(self, filepath):
    if not os.path.exists(filepath):
      open(filepath, 'a').close()

  def _copy_templates(self, source, src, dst):
    env = Environment(loader=PackageLoader('finny.commands', 'templates/' + src))

    for item in source:
      template = env.get_template("%s.jinja" % item)
      output = template.render(name=self.name)

      path = dst + "/" + item
      dirname = os.path.dirname(path)

      if not os.path.exists(dirname):
        os.makedirs(dirname)

      self._touch(dirname + "/__init__.py")

      with open(path, "w+") as f:
        f.write(output)

  def run(self, name, path):
    self.path = path
    self.name = name

    os.makedirs(self.path, 0755)
    os.makedirs(self.path + "/resources", 0755)
    self._touch(self.path + "/resources/__init__.py")

    self.env = Environment(loader=PackageLoader('finny.commands', 'templates'))

    self._copy_templates(BASE_FOLDER_TEMPLATES, "", self.path)

    self._copy_templates(CONFIG_INITIALIZERS_TEMPLATES, "initializers",
                         "%s/%s/initializers" % (self.path, self.name) )

    self._copy_templates(CONFIG_RUNNERS_TEMPLATES, "runners",
                         "%s/%s/runners" % (self.path, self.name) )

    self._copy_templates(CONFIG_TEMPLATES, "config",
                         "%s/%s/" % (self.path, self.name) )
